#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "bitio.c"
#include "main.h"

void usage_exit(char *prog_name);
void print_ratios(char *input,char *output);
long file_size(char *name);
void CompressFile(BIT_FILE *inf, BIT_FILE *ouf, int argc, char *argv[]);

int main(int argc,char *argv[])
{
 BIT_FILE *output;
 BIT_FILE *input;

 setbuf(stdout,NULL);
 if(argc<4)
  usage_exit(argv[0]);

 input = OpenInputBitFile(argv[1]);
 if(input == NULL)
  fatal_error("Error opening %s for input\n",argv[1]);

 output = OpenOutputBitFile(argv[2]);
 if(output == NULL)
  fatal_error("Error opening %s for output\n",argv[2]);

 printf("\nCompressing %s to %s\n",argv[1],argv[2]);
 printf("Using %s\n",CompressionName);

 argc-=3;
 argv+=3;

 CompressFile(input,output,argc,argv);

 argc+=3;		// przywroc poprzednie argumenty do wywolania print_ratios
 argv-=3;

 CloseOutputBitFile(output);
 CloseInputBitFile(input);
 print_ratios(argv[1],argv[2]);
 return(0);
}

void CompressFile(BIT_FILE *inf, BIT_FILE *ouf, int argc, char *argv[])
{
 int i,b,k,m;
 int cnt,x,d;

 k=atoi(argv[0]);	// pobierz argument wywolania programu (liczba k)
 m=pow(2,k);		// m=2^k

 OutputBits(ouf,k,8);	// zapisz do pliku wyjsciowego bajt okreslajacy liczbe k

 b=InputBit(inf);	// pobierz pierwszy bit z pliku wejsciowego
 OutputBit(ouf,b);	// zapisz go do pliku wyjsciowego
 cnt=1;			// licznik odczytanych bajtw na 1

 while (feof(inf->file)==0)	// dopoki nie ma konca pliku
 {
   while((i=InputBit(inf))==b)	// licz kolejne wystapienia tych samych bitow
   {
    if (feof(inf->file))
     break;			// koniec pliku - wyjdz z petli
    cnt++;			// zliczanie kolejnych wystapien takich samych bitow
   }
   b=i;				// bedziemy zliczac nowe bity
   x=cnt>>k;			// podziel - przesun bitowo w lewo (x=cnt/(2^k))
   for (d=0;d<x;d++)
    OutputBit(ouf,1);		// wypisz x jedynek (kod Golomba)
   OutputBit(ouf,0);		// zero konczace kod
   OutputBits(ouf,cnt%m,k);	// wypisz binarnie reszte z dzielenia cnt mod m
   cnt=1;			// juz zliczono nowy bit - ustaw licznik na 1
 }
}

void usage_exit(char *prog_name)
{
 char *short_name;
 char *extension;

 short_name = strrchr(prog_name,'\\');
 if(short_name == NULL)
  short_name = strrchr(prog_name,'/');
 if(short_name == NULL)
  short_name = strrchr(prog_name,':');
 if(short_name != NULL)
  short_name++;
 else
  short_name = prog_name;
 extension = strrchr(short_name,'.');
 if(extension != NULL)
  *extension = '\0';
 printf("\nUsage: %s %s \n",short_name,Usage);
 exit(0);
}

#ifndef SEEK_END
#define SEEK_END 2
#endif

long file_size(char *name)
{
 long eof_ftell;
 FILE *file;

 file = fopen(name,"rb");
 if(file == NULL)
  return(0L);
 fseek(file,0L,SEEK_END);
 eof_ftell = ftell(file);
 fclose(file);
 return(eof_ftell);
}

void print_ratios(char *input,char *output)
{
 long input_size;
 long output_size;
 int ratio;

 input_size = file_size(input);
 if(input_size == 0)
  input_size = 1;
 output_size = file_size(output);
 ratio = 100 - (int)(output_size*100L/input_size);
 printf("\nInput bytes:  %ld\n",input_size);
 printf("Output bytes:  %ld\n",output_size);
 if(output_size == 0)
  output_size = 1;
 printf("Compression ratio:  %d%%\n",ratio);
}
