#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "bitio.c"
//#include "main.h"

#define CompressionName "RLE_Golomb"
#define Usage "plik_wejsciowy plik_wyjsciowy"

void usage_exit(char *prog_name);
void ExpandFile(BIT_FILE *inf, BIT_FILE *ouf, int argc, char *argv[]);

int main(int argc,char *argv[])
{
 BIT_FILE *input;
 BIT_FILE *output;

 setbuf(stdout,NULL);
 if(argc<3)
  usage_exit(argv[0]);

 input = OpenInputBitFile(argv[1]);
 if(input == NULL)
  fatal_error("Error opening %s for input\n",argv[1]);

 output = OpenOutputBitFile(argv[2]);
 if(output == NULL)
  fatal_error("Error opening %s for output\n",argv[2]);

 printf("\nExpanding %s to %s\n",argv[1],argv[2]);
 printf("Using %s\n",CompressionName);

 argc-=3;
 argv+=3;

 ExpandFile(input,output,argc,argv);

 CloseInputBitFile(input);
 CloseOutputBitFile(output);

 putc('\n',stdout);
 return(0);
}

void ExpandFile(BIT_FILE *inf, BIT_FILE *ouf, int argc, char *argv[])
{
 int b,m,k;
 int cnt,d;

 k=InputBits(inf,8);	// pobierz bajt z pliku wejsciowego okreslajacy liczbe k
 m=pow(2,k);		// m=2^k

 b=InputBit(inf);	// pobierz pierwszy bit z pliku wejsciowego
 cnt=0;			// zeruj licznik powtorzen bitow

 while (feof(inf->file)==0)	// dopoki nie ma konca pliku
 {
   while(InputBit(inf)==1)	// licz jedynki (kod Golomba)
   {
    if (feof(inf->file))
     return;			// koniec pliku - wyjdz z funkcji
    cnt++;			// zliczanie kolejnych wystapien '1' w kodzie Golomba
   }

   cnt*=m;			// liczba jedynek w kodzie Golomba * m
   cnt+=InputBits(inf,k);	// odczytaj zapisana reszte z dzielenia przez m i dodaj do cnt

   if (feof(inf->file))
    return;			// jesli osiagnieto koniec pliku - wyjdz (jesli kod skonczyl sie wczesniej)

   for (d=0;d<cnt;d++)
    OutputBit(ouf,b);		// wypisz okreslona ilosc (cnt) bitow do pliku wyjsciowego

   cnt=0;		// zeruj licznik powtorzen bitow
   b=!b & 1;		// zaneguj bit (nastepny bit bedzie przeciwny)
 }
}


void usage_exit(char *prog_name)
{
 char *short_name;
 char *extension;

 short_name = strrchr(prog_name,'\\');
 if(short_name == NULL)
  short_name = strrchr(prog_name,'/');
 if(short_name == NULL)
  short_name = strrchr(prog_name,':');
 if(short_name != NULL)
  short_name++;
 else
  short_name = prog_name;
 extension = strrchr(short_name,'.');
 if(extension != NULL)
  *extension = '\0';
 printf("\nUsage: %s %s \n",short_name,Usage);
 exit(0);
}

